package de.jstacs.sequenceScores.statisticalModels.trainable.latentVariables;

import java.io.BufferedReader;
import java.io.BufferedWriter;

import de.jstacs.data.AlphabetContainer;
import de.jstacs.io.NonParsableException;
import de.jstacs.io.XMLParser;
import de.jstacs.sequenceScores.statisticalModels.trainable.AbstractTrainableStatisticalModel;


/**
 * Abstract class for all models that require latent variables, such as mixture models and promoter models.   
 * @author Ralf Eggeling <eggeling@informatik.uni-halle.de>
 * 
 */

public abstract class AbstractLatentVariableModel extends AbstractTrainableStatisticalModel{

	protected static final String XML_TAG = "AbstractLatentVariableModel";
	static int STOCHASTIC_EM = 0;
	static int GIBBS_SAMPLING = 1;
		
	protected int numberOfInitialIterations;
	
	protected int numberOfAdditionalIterations;
	
	protected int numberOfRestarts;
	
	protected int lengthOfBurnIn;
	
	protected int lagInStationaryPhase;
	
	protected int mode = 0;
	
	protected BufferedWriter writer;
	
	protected BufferedReader reader;
	
	protected String sampleStorageDirectory = "/home/eggeling/output";
	
	public String expString;
	
	/**
	 * Constructor
	 * @param alphabets The alphabet container
	 * @param length The length of the input sequences
	 */
	public AbstractLatentVariableModel(AlphabetContainer alphabets, int length) {
		super(alphabets, length);
	}
	
	/**
	 * Constructor from an XML representation
	 * @param sb The StringBuffer containing the XML representation
	 * @throws NonParsableException
	 */
	public AbstractLatentVariableModel(StringBuffer sb) throws NonParsableException {
		super(sb);
	}
	
	/**
	 * Method for setting a directiory of samples parameter sets, i.e., for writing Gibbs sampling output
	 * @param dir Path
	 */
	public void setSampleStorageDirectory(String dir) {
		sampleStorageDirectory = dir;
	}
	
	/**
	 * Returns the directiory of samples parameter sets, i.e., for reading the Gibbs sampling output
	 * @param dir Path
	 */
	public String getSampleStorageDirectory() {
		return sampleStorageDirectory;
	}

	

}
