package de.jstacs.sequenceScores.statisticalModels.trainable.variableStructure;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;

import de.jstacs.data.AlphabetContainer;
import de.jstacs.data.DataSet;
import de.jstacs.io.NonParsableException;
import de.jstacs.sequenceScores.statisticalModels.trainable.AbstractTrainableStatisticalModel;

/**
 * Abstract class for all trainable statistical models that include a model (structure) selection. 
 * @author Ralf Eggeling <eggeling@informatik.uni-halle.de>
 *
 */
public abstract class AbstractVariableStructureModel extends
		AbstractTrainableStatisticalModel {

	protected BufferedWriter writer;
	
	protected BufferedReader reader;
	
	protected String sampleFilePath;
	
	public int numberOfSamples;
	
	/**
	 * Standard constructor.
	 * @param alphabets The alphabet container.
	 * @param length The sequence length.
	 */
	public AbstractVariableStructureModel(AlphabetContainer alphabets,
			int length) {
		super(alphabets, length);
	}
	
	/**
	 * The standard constructor for the interface {@link de.jstacs.Storable}.
	 * Creates a new {@link AbstractVariableStructureModel} out of a {@link StringBuffer}.
	 * 
	 * @param sb
	 *            the {@link StringBuffer} to be parsed
	 * 
	 * @throws NonParsableException
	 *             is thrown if the {@link StringBuffer} could not be parsed
	 */
	public AbstractVariableStructureModel(StringBuffer sb) throws NonParsableException {
		super(sb);
	}
	
	/**
	 * The function returns the score (AIC, BIC, NML, Bayes) of the currently selected structure. It is interpreted as logarithmic score, i.e., may assume values from -inf to +inf.   
	 * @return structure score
	 */
	public abstract double getLogScoreOfCurrentStructure();
	
	@Override
	public AbstractVariableStructureModel clone() throws CloneNotSupportedException {
		return (AbstractVariableStructureModel) super.clone();
	}
	
	public abstract void parse(String rep);
	
	public abstract String getSparseParameterRepresentation();
	
	public abstract void saveCurrentParametersToFile() throws IOException;
	
	public abstract void initForSampling(String s) throws IOException;
	
	public abstract void drawParameters(DataSet d) throws Exception;
	
	public abstract void samplingStopped() throws IOException;

}