package de.jstacs.sequenceScores.statisticalModels.trainable.variableStructure.parsimonious.dataStructures;
import de.jstacs.data.alphabets.DiscreteAlphabet;
import java.util.TreeSet;

/**
 * 
 * @author Ralf Eggeling <r.eggeling@online.de>
 *
 */
public class PartitionSet {
	
	private TreeSet<Partition> set;
	
	private DiscreteAlphabet alph;
	
	private TreeSet<SymbolSet> symbolset;
	
	public PartitionSet(TreeSet<SymbolSet> symbolset) {
		this.alph=symbolset.first().getAlphabet();
		try {
			this.symbolset=symbolset;
		} catch(Exception e) {
			e.printStackTrace();
		}
		computeValidPartitions();
	}
	
	public PartitionSet(TreeSet<SymbolSet> symbolset, int allowedSubSetSize, int allowedNumberOfExceptions) {
		this.alph=symbolset.first().getAlphabet();
		try {
			this.symbolset=symbolset;
		} catch(Exception e) {
			e.printStackTrace();
		}
		computeValidPartitionsUnderSizeConstraints(allowedSubSetSize, allowedNumberOfExceptions);
	}
	
	/**
	 * computes all valid partitions of this partition set
	 *
	 */
	private void computeValidPartitions() {
		set = new TreeSet<Partition>();
		SymbolSet[] symbolsetarr = new SymbolSet[symbolset.size()];
		symbolset.toArray(symbolsetarr);
		int[] pointer = new int[(int)alph.length()];
		pointer[0]=0;
		for(int i=1;i<pointer.length;i++) {
			pointer[i]=pointer[i-1]+1;
		}
		do {
			Partition part = new Partition(alph,symbolsetarr[pointer[0]]);
			extendPartition(symbolsetarr, part,pointer, 1);
			if(part.isValidPartition()) {
				set.add(part);
//				System.out.println("ADD "+part+"\t"+notIn);
			}
		} while(pointer[0]<symbolsetarr.length);
	}
	
	/**
	 * computes all valid partitions of this partition set
	 *
	 */
	private void computeValidPartitionsUnderSizeConstraints( int allowedSubSetSize, int allowedNumberOfExceptions) {
		set = new TreeSet<Partition>();
		SymbolSet[] symbolsetarr = new SymbolSet[symbolset.size()];
		symbolset.toArray(symbolsetarr);
		int[] pointer = new int[(int)alph.length()];
		pointer[0]=0;
		for(int i=1;i<pointer.length;i++) {
			pointer[i]=pointer[i-1]+1;
		}
		do {
			Partition part = new Partition(alph,symbolsetarr[pointer[0]]);
			extendPartition(symbolsetarr, part,pointer, 1);
			if(part.isValidPartitionUnderSizeConstraint(allowedSubSetSize,allowedNumberOfExceptions)) {
				set.add(part);
//				System.out.println("ADD "+part+"\t"+notIn);
			}
		} while(pointer[0]<symbolsetarr.length);
	}
	
	/**
	 * recursive method for extending a given partial partition
	 * @param arr the array of available symbol sets
	 * @param part the current partition
	 * @param pointer determines which partitions have been already created
	 * @param level The partition already contains level-1 sets of symbols. In this method, the levelth set of symbols is added (if possible)
	 */
	private void extendPartition(SymbolSet[] arr, Partition part, int[] pointer, int level) {
		boolean done = false;
		for(int i=pointer[level];i<arr.length&&!done;i++) {
			if(part.isExtendableBy(arr[i])) {
				part.add(arr[i]);
				pointer[level]=i;
				if(level<pointer.length-1&&!part.isValidPartition()) {
					done = true;
					extendPartition(arr, part,pointer,level+1);
				}					
			}	
		}
		if(!done) {
			pointer[level-1]++;
			for(int j=level;j<pointer.length;j++) {
				pointer[j]=pointer[j-1]+1;
			}
		}
	}
	/*	
	protected void printSymbolSets() {
		Iterator<SymbolSet> it = symbolset.iterator();
		int i = 0;
		while(it.hasNext()) {
			System.out.println(i+" "+it.next().toString());
			i++;
		}
	} */
	
	
	protected TreeSet<SymbolSet> getSymbolSet() {
		return symbolset;
	}
	
	protected TreeSet<Partition> getPartition() {
		return set;
	}
	
	protected DiscreteAlphabet getAlphabet() {
		return alph;
	}
}