package de.jstacs.sequenceScores.statisticalModels.trainable.variableStructure.parsimonious.inhomogeneous;

import de.jstacs.DataType;
import de.jstacs.io.NonParsableException;
import de.jstacs.data.AlphabetContainer;
import de.jstacs.io.XMLParser;
import de.jstacs.parameters.SequenceScoringParameterSet;
import de.jstacs.parameters.SimpleParameter;
import de.jstacs.parameters.validation.NumberValidator;

/**
 * ParameterSet for InhomogeneousPMM class 
 * @author Ralf Eggeling <eggeling@informatik.uni-halle.de
 *
 */
public class InhPMMParameterSet extends SequenceScoringParameterSet<InhomogeneousPMM> {

	
	
	protected static final String XMLTAG="InhomogeneousParsMMParameterSet";
	
	public InhPMMParameterSet(AlphabetContainer con, int length, byte order, double ESS, double treePriorConstant)
			throws Exception {
		super(InhomogeneousPMM.class, con, length, false);
		if(!con.isDiscrete())
			throw new Exception("The model only supports discrete alphabets.");
		parameters.add(new SimpleParameter(DataType.BYTE,"Order","The maximal/initial order of the parsimonious Markov model",true,new NumberValidator<Byte>((byte)0,Byte.MAX_VALUE),(byte)2));
		parameters.add(new SimpleParameter(DataType.DOUBLE,"ESS","The equivalent sample size",true,new NumberValidator<Double>(0d,Double.POSITIVE_INFINITY),1d));
		parameters.add(new SimpleParameter(DataType.DOUBLE,"Structure prior constant","The constant that is used in the structure prior of the context trees",true,new NumberValidator<Double>(Double.NEGATIVE_INFINITY,Double.POSITIVE_INFINITY),0d));
		parameters.get(0).setValue(order);
		parameters.get(1).setValue(ESS);
		parameters.get(2).setValue(treePriorConstant);
	}
	
	public InhPMMParameterSet(StringBuffer sb) throws NonParsableException {
		super(sb);
	}

	public String getInstanceComment() {
		return "The ParameterSet <code>InhParsMMParameterSet</code> is used to initialize <code>InhomogeneousParsimoniousMarkovModel</code>s, which can be trained via Gibbs Sampling.";
	}

	public String getInstanceName() {
		return "InhomogeneousParsimoniousMarkovModel";
	}
	
	public String getXMLTAG	(){
		return XMLTAG;
	}
	
	public StringBuffer toXML() {
		StringBuffer sb = super.toXML();
		XMLParser.addTags(sb, XMLTAG);
		return sb;
	}
		
	public void fromXML ( StringBuffer sb ) throws NonParsableException {
		super.fromXML(XMLParser.extractForTag(sb,getXMLTAG()));
	}

	
	public InhPMMParameterSet clone() throws CloneNotSupportedException  {
		InhPMMParameterSet cl = (InhPMMParameterSet)super.clone();
		//	cl.parameters.get(0) (ParameterList) this.parameters.clone();
		return cl;
	}

}