package de.jstacs.utils.algorithms;
import java.util.Random;

/**
 * This class provides several algorithms for sampling discrete random variables from given distributions.
 * All methods are static and public, so they can be used in arbitrary contexts. 
 * @author Ralf Eggeling <eggeling@informatik.uni-halle.de>
 *
 */


public class BasicSamplingAlgorithms {
	
	/**
	 * Samples a random variable according to a given probability distribution. 
	 * @param prob Probability distribution that is to be sampled from 
	 * @param rand For generating random numbers
	 * @return i with probability prob[i]
	 */
	public static int sampleDiscreteRandomVariable(double[] prob, Random rand) {
		double r = rand.nextDouble();
		double cumulativeProb = 0;
		int res = 0;
		while(cumulativeProb<r) {
			cumulativeProb+=prob[res];
			res++; 
		};
		res--;
		return res;
	}
	
	/**
	 * Samples a random variable according to a given log probability distribution. 
	 * @param logProb prob Probability distribution that is to be sampled from 
	 * @param rand For generating random numbers
	 * @return i with log probability logProb[i]
	 */
	public static int sampleDiscreteRandomVariableFromLogProbs(double[] logProb, Random rand) {
		double r = rand.nextDouble();
		double cumulativeProb = 0;
		int res = 0;
		while(cumulativeProb<r) {
			cumulativeProb+=Math.exp(logProb[res]);
			res++; 
		};
		res--;
		return res;
	}
}
