package de.jstacs.utils.functions;

/**
 * Provides basic combinatorical functions
 * @author Ralf Eggeling <eggeling@informatik.uni-halle.de>
 *
 */
public class Combinatorics {
	
	private Combinatorics() {
		
	}
	
	/**
	 * Computes the binomial coefficient.
	 * @param n the number of data points
	 * @param k the number of categories
	 * @return binomial coefficient n over k
	 */
	public static double binomialCoefficient(int n, int k) {
		if(k>n/2) {
			return binomialCoefficient(n,n-k);
		} else {
			double res = 1;
			for(int i=1;i<=k;i++) {
				res*= n-k+i;
				res/=i;
			}
			return res;
		}
	}
	
	
	
	/**
	 * Computes the faculty of an integer. 	
	 * @param x
	 * @return x!
	 */
	public static int faculty(int x) {
		if(x==0) {
			return 1;
		} else {
			return x*faculty(x-1);
		}
	}
	
}
