package projects.PMMdenovo;

import java.io.File;

import de.jstacs.data.AlphabetContainer;
import de.jstacs.data.DataSet;
import de.jstacs.data.alphabets.DNAAlphabet;
import de.jstacs.io.FileManager;
import de.jstacs.io.StringExtractor;
import de.jstacs.motifDiscovery.SignificantMotifOccurrencesFinder;
import de.jstacs.sequenceScores.statisticalModels.trainable.latentVariables.ZOOPSModel;

public class BindingSitePrediction {
	
	/**
	 * Tool for predicting TFBS based in a set of unaligned (e.g. ChIP-seq positive) based on a previously learned model and a negative data set.  
	 * @author Ralf Eggeling <eggeling@informatik.uni-halle.de>
	 *
	 */
	public static void main(String[] args) throws Exception {
		
		String modelFile = args[0];
		String pos = args[1];
		String neg = args[2];
		
		double significance = 1E-4;
		if(args.length>3) {
			significance = Double.parseDouble(args[3]);
		}
		String out = "bindingsites.txt";
		if(args.length>4) {
			out=args[4];
		}

		
		
		ZOOPSModel model = new ZOOPSModel(FileManager.readFile(modelFile));
		AlphabetContainer con = new AlphabetContainer(DNAAlphabet.SINGLETON);
		char ignore = StringExtractor.USUALLY;
		if(FileManager.readFile(pos).toString().charAt(0)=='>') {
			ignore=StringExtractor.FASTA;
		}
		DataSet positives = new DataSet(con,new StringExtractor(new File(pos),1000,ignore));
		ignore = StringExtractor.USUALLY;
		if(FileManager.readFile(neg).toString().charAt(0)=='>') {
			ignore=StringExtractor.FASTA;
		}
		DataSet control = new DataSet(con,new StringExtractor(new File(neg),1000,StringExtractor.FASTA));
		SignificantMotifOccurrencesFinder smof = new SignificantMotifOccurrencesFinder(model,control,null,significance);
		DataSet bindingSites = smof.getBindingSites(positives, 0);
		StringBuffer sb = new StringBuffer();
		for(int i=0;i<bindingSites.getNumberOfElements();i++) {
			sb.append(bindingSites.getElementAt(i)+"\n");
		}
		FileManager.writeFile(out,sb);
		}
}
