package projects.PMMdenovo;

import java.io.File;

import de.jstacs.classifiers.performanceMeasures.AbstractPerformanceMeasureParameterSet;
import de.jstacs.classifiers.performanceMeasures.PerformanceMeasureParameterSet;
import de.jstacs.classifiers.trainSMBased.TrainSMBasedClassifier;
import de.jstacs.data.AlphabetContainer;
import de.jstacs.data.DataSet;
import de.jstacs.data.alphabets.DNAAlphabet;
import de.jstacs.io.FileManager;
import de.jstacs.io.StringExtractor;
import de.jstacs.results.ResultSet;
import de.jstacs.sequenceScores.statisticalModels.trainable.discrete.homogeneous.HomogeneousMM;
import de.jstacs.sequenceScores.statisticalModels.trainable.discrete.homogeneous.parameters.HomMMParameterSet;
import de.jstacs.sequenceScores.statisticalModels.trainable.latentVariables.ZOOPSModel;
import de.jstacs.sequenceScores.statisticalModels.trainable.variableStructure.parsimonious.dataStructures.ParsimoniousElement.ParameterEstimate;
import de.jstacs.sequenceScores.statisticalModels.trainable.variableStructure.parsimonious.dataStructures.ParsimoniousElement.StructureScore;
import de.jstacs.sequenceScores.statisticalModels.trainable.variableStructure.parsimonious.inhomogeneous.InhPMMParameterSet;
import de.jstacs.sequenceScores.statisticalModels.trainable.variableStructure.parsimonious.inhomogeneous.InhomogeneousPMM;

public class Classification {
	
	/**
	 * Tool for comparing inhomogeneous PMMs of different order within motif discovery based on fragment-based classification of a set of unaligned sequences
	 * (e.g. ChIP-seq positive vs negative) based on a previously learned model.  
	 * @author Ralf Eggeling <eggeling@informatik.uni-halle.de>
	 *
	 */	
	public static void main(String[] args) throws Exception {
		
		int motifWidth = 20;
		int motifOrder = 2;
		int flankingOrder = 2;
		int initialIterationSteps = 50;
		int additionalIterationSteps = 10;
		int restarts = 1;
		AlphabetContainer con = new AlphabetContainer(DNAAlphabet.SINGLETON);
		char ignore = StringExtractor.USUALLY;
		if(FileManager.readFile(args[0]).toString().charAt(0)=='>') {
			ignore=StringExtractor.FASTA;
		}
		DataSet pos_train = new DataSet(con,new StringExtractor(new File(args[0]),1000,ignore));
		
		ignore = StringExtractor.USUALLY;
		if(FileManager.readFile(args[1]).toString().charAt(0)=='>') {
			ignore=StringExtractor.FASTA;
		}
		DataSet neg_train = new DataSet(con,new StringExtractor(new File(args[1]),1000,ignore));
		
		ignore = StringExtractor.USUALLY;
		if(FileManager.readFile(args[2]).toString().charAt(0)=='>') {
			ignore=StringExtractor.FASTA;
		}
		DataSet pos_test = new DataSet(con,new StringExtractor(new File(args[2]),1000,ignore));
		
		
		ignore = StringExtractor.USUALLY;
		if(FileManager.readFile(args[3]).toString().charAt(0)=='>') {
			ignore=StringExtractor.FASTA;
		}
		DataSet neg_test = new DataSet(con,new StringExtractor(new File(args[3]),1000,ignore));

		boolean writeToStdOut = false;
		if(args.length>4) {
			if(!args[4].equals("def")) {
				motifWidth = Integer.parseInt(args[4]);
			}
		}
		if(args.length>5) {
			if(!args[5].equals("def")) {
				motifOrder = Integer.parseInt(args[5]);
			}
		}
		if(args.length>6) {
			if(!args[6].equals("def")) {
				flankingOrder = Integer.parseInt(args[6]);
			}
		}
		if(args.length>7) {
			if(!args[7].equals("def")) {
				initialIterationSteps = Integer.parseInt(args[7]);
			}
		}
		if(args.length>8) {
			if(!args[8].equals("def")) {
				additionalIterationSteps = Integer.parseInt(args[8]);
			}
		}
		if(args.length>9) {
			if(!args[9].equals("def")) {
				restarts = Integer.parseInt(args[9]);
			}
		}
		HomogeneousMM flankingModel = new HomogeneousMM(new HomMMParameterSet(con,1,"",(byte)flankingOrder));
		flankingModel.train(DataSet.union(pos_train,neg_train));
		InhomogeneousPMM motifModel = new InhomogeneousPMM(new InhPMMParameterSet(con, motifWidth, (byte)motifOrder, 1, 0));
		motifModel.setParameterEstimate(ParameterEstimate.FSNML);
		motifModel.setStructureScore(StructureScore.BIC);
		ZOOPSModel model = new ZOOPSModel(motifModel,flankingModel,initialIterationSteps,additionalIterationSteps,restarts);
		model.setTextOutput(writeToStdOut);
		model.train(pos_train);		
		TrainSMBasedClassifier mbc = new TrainSMBasedClassifier(model,flankingModel);
		AbstractPerformanceMeasureParameterSet mp = PerformanceMeasureParameterSet.createFilledParameters( true, 0.99, 0.95, 0.95, 1 );
		ResultSet rs = mbc.evaluate(mp, true, pos_test,neg_test);
		System.out.println(((InhomogeneousPMM)model.getMotifModel()).getNumberOfLeaves()+"\t"+rs.getResultAt(7).getValue());
	}
}
