package projects.PMMdenovo;

import java.io.File;

import de.jstacs.classifiers.performanceMeasures.AbstractPerformanceMeasureParameterSet;
import de.jstacs.classifiers.performanceMeasures.PerformanceMeasureParameterSet;
import de.jstacs.classifiers.trainSMBased.TrainSMBasedClassifier;
import de.jstacs.data.AlphabetContainer;
import de.jstacs.data.DataSet;
import de.jstacs.data.DataSet.PartitionMethod;
import de.jstacs.data.alphabets.DNAAlphabet;
import de.jstacs.io.FileManager;
import de.jstacs.io.NonParsableException;
import de.jstacs.io.StringExtractor;
import de.jstacs.results.ResultSet;
import de.jstacs.sequenceScores.statisticalModels.trainable.discrete.homogeneous.HomogeneousMM;
import de.jstacs.sequenceScores.statisticalModels.trainable.discrete.homogeneous.parameters.HomMMParameterSet;
import de.jstacs.sequenceScores.statisticalModels.trainable.latentVariables.ZOOPSModel;
import de.jstacs.sequenceScores.statisticalModels.trainable.variableStructure.parsimonious.dataStructures.ParsimoniousElement.ParameterEstimate;
import de.jstacs.sequenceScores.statisticalModels.trainable.variableStructure.parsimonious.dataStructures.ParsimoniousElement.StructureScore;
import de.jstacs.sequenceScores.statisticalModels.trainable.variableStructure.parsimonious.inhomogeneous.InhPMMParameterSet;
import de.jstacs.sequenceScores.statisticalModels.trainable.variableStructure.parsimonious.inhomogeneous.InhomogeneousPMM;

public class CrossValidationIteration {
	
	/**
	 * Tool for comparing inhomogeneous PMMs of different order within motif discovery based on fragment-based classification of a set of unaligned sequences
	 * (e.g. ChIP-seq positive vs negative) based on a previously learned model.  
	 * @author Ralf Eggeling <eggeling@informatik.uni-halle.de>
	 *
	 */	
	public static void main(String[] args) throws Exception {
		
		int k = 0;
		int K = 10;
		int motifWidth = 20;
		int motifOrder = 2;
		int flankingOrder = 2;
		int initialIterationSteps = 30;
		int additionalIterationSteps = 30;
		int restarts = 1;
		double structPrior = 0;
		StructureScore structScr = StructureScore.BIC;
		ParameterEstimate paramEst = ParameterEstimate.FSNML;
		AlphabetContainer con = new AlphabetContainer(DNAAlphabet.SINGLETON);
		String dataLocation = "/home/eggeling/data/Uniform-TFBS/sequences/CVsplits"; //args[0];
		String factor = ""; //args[1];
		boolean writeToStdOut = true;
		if(args.length>2) {
			if(!args[2].equals("def")) {
				motifWidth = Integer.parseInt(args[2]);
			}
		}
		if(args.length>3) {
			if(!args[3].equals("def")) {
				motifOrder = Integer.parseInt(args[3]);
			}
		}
		if(args.length>4) {
			if(!args[4].equals("def")) {
				flankingOrder = Integer.parseInt(args[4]);
			}
		}
		if(args.length>5) {
			if(!args[5].equals("def")) {
				if(args[5].equals("MM")) {
					structScr = StructureScore.FNML;
					structPrior=100000;
				} else {
					structScr = StructureScore.parseString(args[5]);
				}
			}
		}
		if(args.length>6) {
			if(!args[6].equals("def")) {
				paramEst = ParameterEstimate.parseString(args[6]);
			}
		}
		if(args.length>7) {
			if(!args[7].equals("def")) {
				initialIterationSteps = Integer.parseInt(args[7]);
			}
		}
		if(args.length>8) {
			if(!args[8].equals("def")) {
				additionalIterationSteps = Integer.parseInt(args[8]);
			}
		}
		if(args.length>9) {
			if(!args[9].equals("def")) {
				restarts = Integer.parseInt(args[9]);
			}
		}
		if(args.length>10) {
			if(!args[10].equals("def")) {
				k = Integer.parseInt(args[10]);
			}
		}
		if(args.length>11) {
			if(!args[11].equals("def")) {
				K = Integer.parseInt(args[11]);
			}
		}
		if(args.length>11) {
			if(!args[11].equals("def")) {
				writeToStdOut = Boolean.parseBoolean(args[11]);
			}
		}
		
		
		DataSet[] positives = new DataSet[K];
		DataSet[] negatives = new DataSet[K];
		
		for(int i=0;i<K;i++) {
			char ignore = StringExtractor.USUALLY;
			if(FileManager.readFile(dataLocation+"/"+factor+"_positives_"+i+".fa").toString().charAt(0)=='>') {
				ignore=StringExtractor.FASTA;
			}
			positives[i] = new DataSet(con,new StringExtractor(new File(dataLocation+"/"+factor+"_positives_"+i+".fa"),1000,ignore));
			
			ignore = StringExtractor.USUALLY;
			if(FileManager.readFile(dataLocation+"/"+factor+"_positives_"+i+".fa").toString().charAt(0)=='>') {
				ignore=StringExtractor.FASTA;
			}
			negatives[i] = new DataSet(con,new StringExtractor(new File(dataLocation+"/"+factor+"_negatives_"+i+".fa"),1000,ignore));
		}
			
		boolean[] boolArr = new boolean[K];
		for(int l = 0;l<K;l++) {
			boolArr[l] = true;
			if(l==k)
				boolArr[l]=false;
		}
		double[] res = performCrossValidationIteration(DataSet.union(positives,boolArr),DataSet.union(negatives,boolArr),positives[k],negatives[k],motifWidth,motifOrder,flankingOrder,structScr,paramEst,initialIterationSteps,additionalIterationSteps,restarts,structPrior,writeToStdOut);
		String result = k+"\t"+res[0]+"\t"+res[1]+"\t"+res[2]+"\n";
		System.out.println(result);
	//	FileManager.writeFile(new File(args[12]+"/"+factor+"_"+k+".txt"), result);		
	}
	
	public static double[] performCrossValidationIteration(DataSet fgTrain, DataSet bgTrain, DataSet fgTest, DataSet bgTest, int motifWidth, int motifOrder, int flankingOrder, StructureScore structScr, ParameterEstimate paramEst, int initialIterationSteps, int additionalIterationSteps, int restarts, double structPrior, boolean writeToStdOut) throws Exception {
		AlphabetContainer con = new AlphabetContainer(DNAAlphabet.SINGLETON);
		HomogeneousMM flankingModel = new HomogeneousMM(new HomMMParameterSet(con,1,"",(byte)flankingOrder));
		flankingModel.train(DataSet.union(fgTrain,bgTrain));
		InhomogeneousPMM motifModel = new InhomogeneousPMM(new InhPMMParameterSet(con, motifWidth, (byte)motifOrder, 1, structPrior));
		motifModel.setParameterEstimate(paramEst);
		motifModel.setStructureScore(structScr);
		ZOOPSModel model = new ZOOPSModel(motifModel,flankingModel,initialIterationSteps,additionalIterationSteps,restarts);
	//	model.setTextOutput(writeToStdOut);
		model.setTextOutput(true);
		model.train(fgTrain);		
		TrainSMBasedClassifier mbc = new TrainSMBasedClassifier(model,flankingModel);
		AbstractPerformanceMeasureParameterSet mp = PerformanceMeasureParameterSet.createFilledParameters( true, 0.99, 0.95, 0.95, 1 );
		ResultSet rs = mbc.evaluate(mp, true, fgTest,bgTest);
		System.out.println(rs.getResultAt(7).getName()+"\t"+rs.getResultAt(8).getName());
		return new double[]{((InhomogeneousPMM)model.getMotifModel()).getNumberOfLeaves(),(Double)rs.getResultAt(7).getValue(),(Double)rs.getResultAt(8).getValue()};
	}

}
