package projects.PMMdenovo;
import java.io.File;
import de.jstacs.data.AlphabetContainer;
import de.jstacs.data.DataSet;
import de.jstacs.data.alphabets.DNAAlphabet;
import de.jstacs.io.FileManager;
import de.jstacs.io.StringExtractor;
import de.jstacs.sequenceScores.statisticalModels.trainable.discrete.homogeneous.HomogeneousMM;
import de.jstacs.sequenceScores.statisticalModels.trainable.discrete.homogeneous.parameters.HomMMParameterSet;
import de.jstacs.sequenceScores.statisticalModels.trainable.latentVariables.ZOOPSModel;
import de.jstacs.sequenceScores.statisticalModels.trainable.variableStructure.parsimonious.dataStructures.ParsimoniousElement.ParameterEstimate;
import de.jstacs.sequenceScores.statisticalModels.trainable.variableStructure.parsimonious.dataStructures.ParsimoniousElement.StructureScore;
import de.jstacs.sequenceScores.statisticalModels.trainable.variableStructure.parsimonious.inhomogeneous.InhPMMParameterSet;
import de.jstacs.sequenceScores.statisticalModels.trainable.variableStructure.parsimonious.inhomogeneous.InhomogeneousPMM;

/**
 * De-novo motif discovery tool for inferring an inhomogeneous PMM from a set of unaligned (e.g. ChIP-seq positive) sequences.  
 * @author Ralf Eggeling <eggeling@informatik.uni-halle.de>
 *
 */
public class ModelTrainer {
	
	public static void main(String[] args) throws Exception {
		int motifWidth = 20;
		int motifOrder = 2;
		int flankingOrder = 2;
		int initialIterationSteps = 50;
		int additionalIterationSteps = 10;
		int restarts = 10;
		AlphabetContainer con = new AlphabetContainer(DNAAlphabet.SINGLETON);
		char ignore = StringExtractor.USUALLY;
		if(FileManager.readFile(args[0]).toString().charAt(0)=='>') {
			ignore=StringExtractor.FASTA;
		}
		DataSet data = new DataSet(con,new StringExtractor(new File(args[0]),1000,ignore));
		String outputfilename = "model";
		
		if(args.length>1) {
			if(!args[1].equals("def")) {
				motifWidth = Integer.parseInt(args[1]);
			}
		}
		if(args.length>2) {
			if(!args[2].equals("def")) {
				motifOrder = Integer.parseInt(args[2]);
			}
		}
		if(args.length>3) {
			if(!args[3].equals("def")) {
				flankingOrder = Integer.parseInt(args[3]);
			}
		}
		if(args.length>4) {
			if(!args[4].equals("def")) {
				initialIterationSteps = Integer.parseInt(args[4]);
			}
		}
		if(args.length>5) {
			if(!args[5].equals("def")) {
				additionalIterationSteps = Integer.parseInt(args[5]);
			}
		}
		if(args.length>6) {
			if(!args[6].equals("def")) {
				restarts = Integer.parseInt(args[6]);
			}
		}
		if(args.length>7) {
			outputfilename = args[7];
		}

		HomogeneousMM flankingModel = new HomogeneousMM(new HomMMParameterSet(con,1,"",(byte)flankingOrder));
		flankingModel.train(data);
		InhomogeneousPMM motifModel = new InhomogeneousPMM(new InhPMMParameterSet(con, motifWidth, (byte)motifOrder, 1, 0));
		motifModel.setParameterEstimate(ParameterEstimate.FSNML);
		motifModel.setStructureScore(StructureScore.BIC);
		ZOOPSModel model = new ZOOPSModel(motifModel,flankingModel,initialIterationSteps,additionalIterationSteps,restarts);
		model.setTextOutput(false);
		model.train(data);
		
		FileManager.writeFile(new File(outputfilename+".xml"),model.toXML());
		FileManager.writeFile(new File(outputfilename+".dot"),((InhomogeneousPMM)model.getMotifModel()).getGraphViz());
	}
}
